###########################################
#
# Script for Modeling SDMs in Class
# Written by Noah Charney, Spring 2021
#		for Advanced Conservation Biology
#		University of Maine Orono
#	Modified for Charney ND, SantaLucia K, Miles M, 
#		Adelman I, and Bolton Jr, D. 2024
#		Teaching Issues and Experiments in Ecology
#
###########################################

# -A- INTRODUCTION TO R
#
#	Run each line of code in this file in order.  
# This script should be open in RStudio.
# the number sign "#" at the beginning of this line means it is just a note
# Place your cursor on the line below, then hit "Ctrl" and "Enter" together

     3+5

# Hopefully something happened in the Console Window
# Alternatively, you could click the "Run" button 
# Now try each of these lines in order

     stored_number <- 15 + 30
     stored_number
     stored_number - 40
     stored_number * 10
     stored_number

# Notice that we just created a variable, the name is up to us.

     A <- 20
     B <- 3
     A + B
     silly_named.variable <- 100
     silly_named.variable + A

# Sometimes graphs will pop up when you run something, in the "Plots" tab, 
# probably to the right on your screen.
# Try this:

     plot(1:100,sample(100,100))

     
# -B- Setting up the workspace for Today's lab
#
# Today, we are Running an SDM (Species Distribution Model).
# First, run this line below to clear the workspace of anything that might
# have been saved last time someone ran R on this machine
rm(list = ls())

# First, you need to install and load the R packages we will be using:
install.packages(c('dismo','sf'))
library(dismo)
library(sf)

# Then set working directory with the toolbar menu.  Go to:
#   Session
#       -> Set Working Directory
#             ->  To Source File Location


# Next, read in the source R file.  
# Advanced users could open & look through this file.
source('SDM_source.R')

# -C- Load the data we will be using in our model

# Read in the US Forest Inventory Analysis (FIA) program data. 
FIA <- read.csv('FIA_northeast.csv')

# The FIA provides data on the distribution and condition of America's forests 
# in every state. The program monitors over 29,000 permanent field inventory plots. 
# Individual plots are resampled every 5-10 years for trees species, tree size,
# tree condition, and more. This data will be used to create maps of the starting
# condition for a species distribution model.

# Have a peak at the first 6 rows of it:
head(FIA)
# Peak at the last 6 rows of it:
tail(FIA)

# The output below will give you the species to choose from and the associated 
# number of FIA points where that species was observed in our data.
# You'll want a species that occurs in the Northeast and has a good number of points.
# Just look at this for now, you'll need to come back to it later.
sort(table(FIA[,'common_name']))

# Read in the current climate, consisting of 19 "Bioclim" variables
bioclim.cur <- stack('bioclim_cur_northeast')

#Bioclim variables are temperature and rainfall measures that provide biologically
#relevant information for ecological modeling, such as annual trends, seasonality
#and, extreme factors. These trends include mean annual temperature, annual
#precipitation, and temp/precipiation range.
#See list of all of the bioclims in the handout.

#Look first at just Bio_1 (Mean Temperature) -- see plot window
plot(bioclim.cur[[1]])  #just change the "1" to another number for another variable

# Then Bio_12 (Mean Precip)
plot(bioclim.cur[[12]])

# Then many Bioclim variables at once:
plot(bioclim.cur)

# -D- Parameterizing your model
#
#	Below is where you will be playing with the model
#	First, just use the parameters written here without changing
#	After you have run through it all once, iteratively change the inputs, 
# each time coming back to this point and re-run through the model again.

# Choose a species to model, refer back to line 89 above.
species <- 'eastern hemlock'  #enter a species name, EXACTLY as in table

# Next choose a future global emissions scenario. This activity provides the choice
# between four different future emissions scenarios. SSP1 "Taking the Green"
# represents a continuous global effort to increase sustainability to keep us on 
# track with the boundaries set by the ICPP. SSP2 "Middle of the Road" represents
# a business as usual approach. SSP 3 "A Rocky Road" represents a lack of coordinated
# efforts and severe environmental degredation. SSP 5 "Taking the Highway" represents
# a future with a massive increase in global emissinos.

SSP <- 2      #enter a future scenario, either 1, 2, 3, or 4

# Finally, choose a General Circulation Model. GCMs are used to study climate 
# processes and variability. In this activity, the GCM of your choice will incorporate 
# the bioclim variables and the emissions scenarios you select as additional 
# parameters for predicting future climate conditions in our study area.

GCM <- 1      #enter a General Circulation Model, either 1 or 2

# Choose a future year you want to model, note the data is in 20-year increments.
future.year <- 2070 #enter a year between 2021 and 2100

# Choose the bioclim variables you want to add as paramaters for your GCM.
bioclims <- c(1, 10, 18)  # inside the parentheses, enter 2 or more numbers 
													# between 1 and 19, with commas
                          # if you want all 19 bioclims, enter (1:19)

# -E- Run the SDMs.
#
# The code relies on the bioclim() function in the dismo package.  To learn about
# this SDM algorithm, run this line below, and the help file should appear with 
# a description in the "Help" tab, probably to the right on your screen:
?bioclim

# Plots should show up in a separate window on your desktop (if using PC).
# The images will also save in the "Figures/" folders in working directory.
# In the output maps: 
#    Each black circle is an FIA plot where the species was actually observed 
#    The colors represent the modeled prediction of how suitable the climate 
#            is for that species based on the historical data.  
# Note, depending on your data and your computer speed, the lines below
#	might take a few minutes to run, so be patient.

# First, choose a state & look up close to see how well the SDM matches actual points:
focal.state <- 'ME' #if doing Northeast region, choose from any of these: CT, MA, ME, NH, NJ, NY, OH, PA, RI, VT
sdm.state(focal.state)

#Then look at your future scenario across the larger region:
sdm()

# Now, adjust the parameters above (lines 116 - 142) and 
# run the sdm.state() and sdm() functions again to answer questions on your handout.


# -F- Try out a new region
#
# The function below will subset a new regional focus of your choice.
# To work, you must have downloaded (or your instructor) the "global" folder
# and placed inside the "SDM" folder.  This is a large folder, which is why
# it isn't included by default.
# Below, it is set to create a region we called "southeast" with the 7 states listed.
# You can create any region you want, named whatever you want
# using any set of state codes, including US territories of AS, GU, PR, and VI.
#	Note: because Alaska wraps around the other side of the world, 
# including Alaska your region will yield maps that include most of the world.
# Once you run this, you can modify your code above, replacing "northeast"
# with your new region name on lines 72 and 92 above 
#   e.g. "FIA <- read.csv('FIA_southeast.csv')" and "bioclim.cur <- stack('bioclim_cur_southeast')"
# then start back up at the beginning of section C (line 69)
# IMPORTANT: after setting your region below, if you re-run line 67 above, source('SDM_source.R'), 
# that will reset the region variable to 'northeast', so you will need to then
# reset it to what you want on line 201 below.  We recommend just skipping the source line
# This may take a long time to run, depending on your machine, but should give 
# you a time estimate after getting about 5% of the way through
create.region(region='southeast', f_states=c('GA','TN','AL','NC','SC','MS','FL'))

#To see all your state code options, or just the lower 48 states run these lines:
state_codes
lower48
#you could use one of those vectors, or subsets of them as input in the create.region() function above

# Note, the create.region() function above sets the global environmental variable for region, 
#  but does not delete the data from your previous regions.  To set your region
#  back to northeast, you can do so like this:
region <- 'northeast'