###########################################
#
# Source Code for Modeling SDMs in Class
# Written by Noah Charney, Spring 2021
#		for Advanced Conservation Biology
#		University of Maine Orono
#	Modified for Charney ND, SantaLucia K, Miles M, 
#		Adelman I, and Bolton Jr, D. 2024
#		Teaching Issues and Experiments in Ecology
#
###########################################
#set global variable environmental for region.
#Instructors: if you want to default to a region you create, set it here:
region <<- 'northeast' 

#loads an object with all state abbreviations
load('state_codes')
load('lower48')

#call the sdm() function modified to do a single state
sdm.state <- function(focal.state='ME'){
  sdm(state.fitting = TRUE)
}

#this is the main sdm function
sdm<- function(state.fitting=FALSE){
  lonlat <- data.frame(FIA[,c('LON','LAT')])

  #cleans up species names (removes special characters and sets to lower case for matching)
  f.species <- tolower(gsub('[^[:alnum:]]','',species))
  fia.species <- tolower(gsub('[^[:alnum:]]','',FIA[,'common_name']))

  #gets the coordinates for the points where the focal species occurred  
  focal.lonlat <- lonlat[fia.species==f.species,]
  
  #these are all the options to choose from
  all.models <- c('MIROC6','IPSL-CM6A-LR')
  all.times <- c('2021-2040','2041-2060','2061-2080','2081-2100')
  all.ssp <- c('ssp126','ssp245','ssp370','ssp585') 
  #note, if you don't like that SSP <- 4 gives you ssp585, you could do something like:
  #all.ssp <- c('ssp126','ssp245','ssp370',NA,'ssp585') #gives an error
  #or
  #all.ssp <- c('ssp126','ssp245','ssp370','ssp585','ssp585') #this way using either 4 or 5 will do the same thing

  mid.times <- c(2030,2050,2070,2100)
  t <- which.min(abs(mid.times - future.year)) #match the future year chosen by user to the closest decade
  fut.file <- paste('WorldClim/',region,'/wc2.1_2.5m_bioc_',all.models[GCM],'_', all.ssp[SSP],'_', all.times[t],'.tif',sep='')  
  bioclim.fut <- stack(fut.file) #load the correct future climate layers given the year/SSP/CCM combo
  names(bioclim.fut) <- names(bioclim.cur)
  
  bioclim.out <- bioclim(bioclim.cur[[bioclims]],p=focal.lonlat) #fit the SDM
  bc.predict <- predict(bioclim.out,bioclim.cur[[bioclims]]) #make predictions with the SDM to the current
  bc.predict.fut <- predict(bioclim.out,bioclim.fut) #make predictions with the SDM to the future
  
  focal.shp <- st_read(paste0(region,'.shp')) #read in shapefile of all state boundaries
  focal.state.shp <- subset(focal.shp, STATE == focal.state) #get just the focal state
  
  if(Sys.info()[1] == "Windows"){ windows() } #if not on a windows machine this may throw an error
  plot.new()
  if(state.fitting){
  	#state fitting is just whether you plot at the state level to check fit or not
  	# we're not actually fitting a different model
    par(mfrow=c(1,1)) #set the number of plotting frames
    par(mai=c(.5,.5,.8,1)) #set the margin in inches
    plot(st_geometry(focal.state.shp), #plot the state borders
    		 main=paste(species,' - Current'))
    plot(bc.predict,add=TRUE) #plot the SDM prediction
    plot(st_geometry(focal.state.shp),add=TRUE)
    points(focal.lonlat,col=rgb(0,0,0,.5)) #plot the FIA points
    
    x0 <- par('usr')[1] #get window size for saving image
    mtext(side=3,at=x0,adj=0,paste('Bioclims:',paste(bioclims,collapse = ', ')))
    
    file.name <- paste('Figures/',species,'_BC',paste(bioclims,collapse = '-'),'_',focal.state,'_current.jpg',sep='')
    dir.create('Figures',showWarnings=FALSE)
    dev.print(jpeg, file = file.name, width = dev.size()[1], units='in',res=600) #save image
    
  }else{
    par(mfrow=c(2,1)) #set two frames in the plotting window
  	par(omi=c(0.2,0,.4,0)) #outer margins in inches
    par(mai=c(.2,.4,.5,.2)) #inner margin in inches
    plot(bc.predict) #plot SDM prediction
    plot(st_geometry(focal.shp),add=TRUE) #plot state boundaries
    x0 <- par('usr')[1] #get frame dimentions for saving image
    x1 <- par('usr')[2]
    bioclim.text <- paste('Bioclims:',paste(bioclims,collapse = ', ')) #create text for annotating plot
    mtext(outer=TRUE,side=3,cex=1.5,padj=-.5,species) #write text in margins
    mtext(outer=TRUE,side=3,cex=1,padj=1,bioclim.text)
  	mtext(side=3,at=x0,adj=0,'Current')
    
    plot(bc.predict.fut) #plot the prediction
    plot(st_geometry(focal.shp),add=TRUE) #plot the political boundaries
    
    ssp.names <- c('SSP1-RCP2.6','SSP2-RCP4.5','SSP3-RCP7.0','SSP5-RCP8.5')
    future.text <- paste('Years: ',all.times[t],', Scenario: ',ssp.names[SSP],', GCM: ',all.models[GCM],sep='')
    mtext(side=3,at=x0,adj=0,future.text)
    
    file.name <- paste('Figures/',species,'_BC',paste(bioclims,collapse = '-'),'_',all.times[t],'_',all.ssp[SSP],'_',all.models[GCM],'.jpg',sep='')
    dir.create('Figures',showWarnings=FALSE)
    dev.print(jpeg, file = file.name, width = dev.size()[1], units='in',res=600)
  }
}


create.region <- function(region, f_states,overwrite=FALSE){
	#This Function sets up the space for running an SDM in your region by creating:
	#		a subset of FIA points,
	#		a subset of state boundaries for mapping
	#		a subset of current and future bioclims for modeling
	#Arguments:
	#   region: a character giving the name of your region
	#   f_states: a vector of the 2-letter state code abbreviations for the members of your region
	#   overwrite: logical, whether or not to overwrite an existing region of the same name
	
	if(file.exists(paste0("FIA_",region,".csv")) & !overwrite){
		stop('Region already exists!  Use ovewrite=TRUE or try a different name.')
	}
	
	#create regional shapefile of states
  us_shp <- st_read('global/US_Canada.shp')
  f_state_shp <- us_shp[us_shp$STATE %in% f_states,]
  if(overwrite){
	  st_write(f_state_shp, paste0(region,".shp"),append=!overwrite)
  }else{
  	 st_write(f_state_shp, paste0(region,".shp"))
  }
  
  #create regional subset of FIA data
  FIA_allstates <- read.csv("global/FIA_allstates.csv")
  focal_FIA <- FIA_allstates[FIA_allstates[,'state'] %in% f_states,]
  write.csv(focal_FIA, paste0("FIA_",region,".csv"), row.names=FALSE)
  
  #create regional subset bioclims
  start_time <- proc.time()[3]
	bioclim_cur_global <- stack('global/bioclim_cur_global')
  bioclim_cur_focal <- crop(bioclim_cur_global,extent(f_state_shp))
  bioclim_cur_focal <- mask(bioclim_cur_focal,f_state_shp)
  writeRaster(bioclim_cur_focal,filename=paste0('bioclim_cur_',region),overwrite=overwrite)
  time_one <- proc.time()[3] - start_time	

  all_files <- list.files('global/worldclim/')
  all_files <- all_files[substr(all_files,1,3) == 'wc2']
  dir.create(paste0('worldclim/',region))
  for(i in 1:length(all_files)){
  	print(paste(round(((length(all_files) - i) * time_one)/60,1),'minutes remaining'))
  	start_time <- proc.time()[3]
  	fut_rast_focal <- stack(paste0('global/worldclim/',all_files[i]))
  	fut_rast_focal <- crop(fut_rast_focal,extent(f_state_shp))
 	  fut_rast_focal <- mask(fut_rast_focal,f_state_shp)
  	writeRaster(fut_rast_focal,filename=paste0('worldclim/',region,'/',all_files[i]),overwrite=overwrite)
  	time_one <- proc.time()[3] - start_time	
  }
 	#reset global environmental variable for region
	region <<- region
	
	#tell the user:
	print(paste0('Region set to: ',region))
}